/**
 * JASMINe
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Activator.java 2693 2008-10-30 13:12:22Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi;

import java.io.File;
import java.util.HashMap;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.ow2.jasmine.rules.AbstractDroolsWorkingMemory;
import org.ow2.jasmine.rules.api.IDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IOSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;
import org.ow2.jasmine.rules.osgi.impl.OSGiDroolsWorkingMemory;
import org.ow2.jasmine.rules.osgi.impl.PackageAdderListener;
import org.ow2.jonas.lib.bootstrap.LoaderManager;


/**
 * Start and stop the Drools working memory.
 * @author Loris Bouzonnet
 */
public class Activator implements BundleActivator {

    /**
     * Keep a trace of the registration service in order to unregister it before stopping.
     */
    private ServiceRegistration serviceRegistration;

    /**
     * The Drools working memory.
     */
    private IDroolsWorkingMemory droolsWorkingMemory;

    /**
     * Listener to dynamically add and remove rules packages.
     */
    private ServiceListener packageAdderListener;

    /**
     * Start the working memory and register it in the given {@link BundleContext}.
     * @param context the context to register the working memory
     * @throws Exception if the working memory cannot be started
     */
    public void start(final BundleContext context) throws Exception {

        // Retrieve the already registered rules packages
        ServiceReference[] packageAdderRefs =
            context.getAllServiceReferences(IPackageAdder.class.getName(), null);

        HashMap<ServiceReference, IPackageAdder> packageAdders =
            new HashMap<ServiceReference, IPackageAdder>();
        if(packageAdderRefs != null) {
            for(ServiceReference packageAdderRef : packageAdderRefs) {
                packageAdders.put(packageAdderRef,
                        (IPackageAdder) context.getService(packageAdderRef));
            }
        }

        // FIXME Use an other classloader
        // Bad... but required to compile Java files in drools-compiler (this using Eclipse or Janino).
        // Required for the java dialect, but not for mvel (mvel avoids compilation and seems better :).
        droolsWorkingMemory = new OSGiDroolsWorkingMemory(
                LoaderManager.getInstance().getExternalLoader(), packageAdders);

        // In development, dump drools generated files
        if(Boolean.getBoolean("jonas.developer")) {
            File dumpDir = new File(System.getProperty("java.io.tmpdir") + File.separator + "drools_dump");
            boolean exists = dumpDir.exists();
            if(!exists) {
                exists = dumpDir.mkdir();
            }
            if(exists && dumpDir.isDirectory()) {
                ((AbstractDroolsWorkingMemory) droolsWorkingMemory).setDumpDir(dumpDir);
            }
        }

        // Start the working memory
        droolsWorkingMemory.start();

        // Add a listener for adding new packages to the working memory
        packageAdderListener = new PackageAdderListener(context, (IOSGiDroolsWorkingMemory) droolsWorkingMemory);
        context.addServiceListener(packageAdderListener, "(objectClass=" + IPackageAdder.class.getName() + ")");

        // Register it
        serviceRegistration =
            context.registerService(IDroolsWorkingMemory.class.getName(),
                    droolsWorkingMemory, null);
    }

    /**
     * Unregister the working memory of the given {@link BundleContext} and stop it.
     * @param context the context to unregister the working memory
     */
    public void stop(final BundleContext context) {

        // Unregister the working memory
        if(serviceRegistration != null) {
            serviceRegistration.unregister();
        }

        // Remove the package listener
        if(packageAdderListener != null) {
            context.removeServiceListener(packageAdderListener);
        }

        // Stop it
        if(droolsWorkingMemory != null) {
            droolsWorkingMemory.stop();
        }

    }

}
